{
  FCEW.PAS : Wrapper for FCE.PAS

  The functions in this unit call the functions in the FCE.PAS unit.
  Note that normal Delphi strings are passed to all functions in this
  unit, rather than PChar pointers, as in the FCE.PAS unit. Also note
  that Delphi 1 (unlike later versions) is limited to 255 byte strings.

  See example program FTPW, which uses this unit.
}

unit fcew;

interface

uses fce, SysUtils;

function fAbort(fceChan:Integer):Integer;
function fAttach(fceNbrChans:Integer;fceKeyCode:LongInt):Integer;
function fClose(fceChan:Integer):Integer;
function fCommand(fceChan:Integer;fceCmd:String):Integer;
function fConnect(fceChan:Integer;fceServer:String;fceUser:String;fcePass:String):Integer;
function fDebug(fceChan:Integer;fceValue:Integer;var fceBuffer:String):Integer;
function fDelFile(fceChan:Integer;fceFileName:String):Integer;
function fDelServerDir(fceChan:Integer;fceServerName:String):Integer;
function fDriver(fceChan:Integer):Integer;
function fErrorText(fceChan:Integer;fceCode:Integer;var fceBuffer:String):Integer;
function fExtract(fceBuffer:String;fceLineNbr:Integer;fceFieldNbr:Integer;var fceBufPtr:String):Integer;
function fGetFile(fceChan:Integer;fceFileName:String):Integer;
function fGetInteger(fceChan:Integer;fceParamName:Integer):LongInt;
function fGetList(fceChan:Integer;fceListType:Integer;var fceBuffer:String):Integer;
function fGetLocalDir(fceChan:Integer;var fceBuffer:String):Integer;
function fGetServerDir(fceChan:Integer;var fceBuffer:String):Integer;
function fGetString(fceChan:Integer;fceParamName:Integer;var fceBuffer:String):Integer;
function fHello(fceChan:Integer):Integer;
function fMakeServerDir(fceChan:Integer;fceServerDir:String):Integer;
function fPutFile(fceChan:Integer;fceFileName:String):Integer;
function fRelease:Integer;
function fSetInteger(fceChan:Integer;fceParamName:Integer;fceParamValue:LongInt):Integer;
function fSetLocalDir(fceChan:Integer;fceDirName:String):Integer;
function fSetMode(fceChan:Integer;fceXferMode:Char):Integer;
function fSetServerDir(fceChan:Integer;fceDirName:String):Integer;
function fSetString(fceChan:Integer;fceParamName:Integer;fceParamString:String):Integer;

implementation

function fAbort(fceChan:Integer):Integer;
begin
  fAbort := fceAbort(fceChan)
end;

function fAttach(fceNbrChans:Integer;fceKeyCode:LongInt):Integer;
begin
  fAttach := fceAttach(fceNbrChans,fceKeyCode)
end;

function fClose(fceChan:Integer):Integer;
begin
  fClose := fceClose(fceChan)
end;

function fCommand(fceChan:Integer;fceCmd:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := Length(fceCmd)+1;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceCmd);
  {make the call}
  Code := fceCommand(fceChan, StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fCommand := Code
end;

function fConnect(fceChan:Integer;fceServer:String;fceUser:String;fcePass:String):Integer;
var
  Code    : Integer;
  HostLen : Integer;
  HostPtr : PChar;
  UserLen : Integer;
  UserPtr : PChar;
  PassLen : Integer;
  PassPtr : PChar;
begin
  {allocate memory for fceServer}
  HostLen := Length(fceServer)+1;
  GetMem(HostPtr, HostLen);
  StrPCopy(HostPtr, fceServer);
  {allocate memory for fceUser}
  UserLen := Length(fceUser)+1;
  GetMem(UserPtr, UserLen);
  StrPCopy(UserPtr, fceUser);
  {allocate memory for fcePass}
  PassLen := Length(fcePass)+1;
  GetMem(PassPtr, PassLen);
  StrPCopy(PassPtr, fcePass);
  {make the call}
  Code := fceConnect(fceChan, HostPtr, UserPtr, PassPtr);
  {free memory}
  FreeMem(HostPtr, HostLen);
  FreeMem(UserPtr, UserLen);
  FreeMem(PassPtr, PassLen);
  {return result}
  fConnect := Code
end;

function fDebug(fceChan:Integer;fceValue:Integer;var fceBuffer:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := 128;
  GetMem(StringPtr, StringLen);
  {make the call}
  Code := fceDebug(fceChan, fceValue, StringPtr, StringLen);
  fceBuffer := StrPas(StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fDebug := Code
end;

function fDelFile(fceChan:Integer;fceFileName:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := Length(fceFileName)+1;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceFileName);
  {make the call}
  Code := fceDelFile(fceChan, StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fDelFile := Code
end;

function fDelServerDir(fceChan:Integer;fceServerName:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := Length(fceServerName)+1;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceServerName);
  {make the call}
  Code := fceDelServerDir(fceChan, StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fDelServerDir := Code
end;

function fDriver(fceChan:Integer):Integer;
begin
  fDriver := fceDriver(fceChan)
end;

function fErrorText(fceChan:Integer;fceCode:Integer;var fceBuffer:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := 128;
  GetMem(StringPtr, StringLen);
  {make the call}
  Code := fceErrorText(fceChan, fceCode, StringPtr, StringLen);
  fceBuffer := StrPas(StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fErrorText := Code
end;

function fExtract(fceBuffer:String;fceLineNbr:Integer;fceFieldNbr:Integer;var fceBufPtr:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
  BufferLen : Integer;
  BufferPtr : PChar;
begin
  {allocate source memory}
  StringLen := Length(fceBuffer)+1;
  GetMem(StringPtr, StringLen);
  {allocate target memory}
  BufferLen := 5000;
  GetMem(BufferPtr, BufferLen);
  {make the call}
  Code := fceExtract(StringPtr, fceLineNbr, fceFieldNbr, BufferPtr, BufferLen);
  fceBuffer := StrPas(StringPtr);
  FreeMem(StringPtr, StringLen);
  FreeMem(BufferPtr, BufferLen);
  {return result}
  fExtract := Code
end;

function fGetFile(fceChan:Integer;fceFileName:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := Length(fceFileName)+1;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceFileName);
  {make the call}
  Code := fceGetFile(fceChan, StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fGetFile := Code
end;

function fGetInteger(fceChan:Integer;fceParamName:Integer):LongInt;
begin
  fGetInteger := fceGetInteger(fceChan, fceParamName);
end;

function fGetList(fceChan:Integer;fceListType:Integer;var fceBuffer:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := 15000;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceBuffer);
  {make the call}
  Code := fceGetList(fceChan, fceListType, StringPtr, StringLen);
  fceBuffer := StrPas(StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fGetList := Code
end;

function fGetLocalDir(fceChan:Integer;var fceBuffer:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := 128;
  GetMem(StringPtr, StringLen);
  {make the call}
  Code := fceGetLocalDir(fceChan, StringPtr, StringLen);
  fceBuffer := StrPas(StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fGetLocalDir := Code
end;

function fGetServerDir(fceChan:Integer; var fceBuffer:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := 128;
  GetMem(StringPtr, StringLen);
  {make the call}
  Code := fceGetServerDir(fceChan, StringPtr, StringLen);
  fceBuffer := StrPas(StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fGetServerDir := Code
end;

function fGetString(fceChan:Integer;fceParamName:Integer;var fceBuffer:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := 128;
  GetMem(StringPtr, StringLen);
  {make the call}
  Code := fceGetString(fceChan, fceParamName, StringPtr, StringLen);
  fceBuffer := StrPas(StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fGetString := Code
end;

function fHello(fceChan:Integer):Integer;
begin
  fHello := fceHello(fceChan)
end;

function fMakeServerDir(fceChan:Integer;fceServerDir:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := Length(fceServerDir)+1;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceServerDir);
  {make the call}
  Code := fceMakeServerDir(fceChan, StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fMakeServerDir := Code
end;

function fPutFile(fceChan:Integer;fceFileName:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := Length(fceFileName)+1;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceFileName);
  {make the call}
  Code := fcePutFile(fceChan, StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fPutFile := Code
end;

function fRelease:Integer;
begin
  fRelease := fceRelease
end;

function fSetInteger(fceChan:Integer;fceParamName:Integer;fceParamValue:LongInt):Integer;
begin
  fSetInteger := fceSetInteger(fceChan, fceParamName, fceParamValue)
end;

function fSetLocalDir(fceChan:Integer;fceDirName:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := Length(fceDirName)+1;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceDirName);
  {make the call}
  Code := fceSetLocalDir(fceChan, StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fSetLocalDir:= Code
end;

function fSetMode(fceChan:Integer;fceXferMode:Char):Integer;
begin
  fSetMode := fceSetMode(fceChan, fceXferMode);
end;

function fSetServerDir(fceChan:Integer;fceDirName:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := Length(fceDirName)+1;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceDirName);
  {make the call}
  Code := fceSetServerDir(fceChan, StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fSetServerDir := Code
end;

function fSetString(fceChan:Integer;fceParamName:Integer;fceParamString:String):Integer;
var
  Code      : Integer;
  StringLen : Integer;
  StringPtr : PChar;
begin
  {allocate memory}
  StringLen := Length(fceParamString)+1;
  GetMem(StringPtr, StringLen);
  StrPCopy(StringPtr, fceParamString);
  {make the call}
  Code := fceSetString(fceChan, fceParamName, StringPtr);
  FreeMem(StringPtr, StringLen);
  {return result}
  fSetString := Code
end;

(* initialization *)
begin
end.
