unit Ftp2_pgm;

interface

uses
  fce, keycode, displayu,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls;

type
  TWinFTP = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    eServer: TEdit;
    eUser: TEdit;
    ePass: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    eLocalDir: TEdit;
    Label7: TLabel;
    Label8: TLabel;
    eServerDir: TEdit;
    Label9: TLabel;
    bConnect: TButton;
    bExit: TButton;
    bLocalDir: TButton;
    bServerDir: TButton;
    bName: TButton;
    bFull: TButton;
    Label10: TLabel;
    oAscii: TRadioButton;
    oBinary: TRadioButton;
    Label12: TLabel;
    eFilename: TEdit;
    Label13: TLabel;
    bGet: TButton;
    bPut: TButton;
    bDel: TButton;
    bChk: TButton;
    eMessage: TMemo;
    bClear: TButton;
    Label11: TLabel;
    Timer1: TTimer;
    eStatus: TEdit;
    procedure bConnectClick(Sender: TObject);
    procedure bExitClick(Sender: TObject);
    procedure bClearClick(Sender: TObject);
    procedure bFullClick(Sender: TObject);
    procedure bNameClick(Sender: TObject);
    procedure bLocalDirClick(Sender: TObject);
    procedure bServerDirClick(Sender: TObject);
    procedure oAsciiClick(Sender: TObject);
    procedure oBinaryClick(Sender: TObject);
    procedure bGetClick(Sender: TObject);
    procedure bPutClick(Sender: TObject);
    procedure bDelClick(Sender: TObject);
    procedure bChkClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }

  end;

const
   BUFFER_SIZE = 15000;

var
  WinFTP: TWinFTP;

implementation

{$R *.DFM}

procedure TWinFTP.bConnectClick(Sender: TObject);
var
  Code : Integer;
  Build : Integer;
  D1, D2, D3 : Integer;
  Text : String;
  ServerPtr  : PChar;
  UserPtr : PChar;
  Passptr : PChar;
  TempPtr : PChar;
begin
  {use bExit as "connected" flag}
  if not bExit.Enabled then
    begin
      {close existing connection}
      fceClose(0);
      fceRelease;
      {ready for new connection}
      bConnect.Caption := 'CONNECT';
      bExit.Enabled := True;
    end
  else
    begin
      {do we have all necessary parameters ?}
      if Length(eServer.Text) = 0 then
      begin
        DisplayLine(eMessage,'FTP server name is missing');
        exit;
      end;
      if Length(eUser.Text) = 0 then
      begin
        DisplayLine(eMessage,'FTP user name is missing');
        exit;
      end;
      if Length(ePass.Text) = 0 then
      begin
        DisplayLine(eMessage,'FTP password is missing');
        exit;
      end;
      {attach FCE}
      Code := fceAttach(1, FCE_KEY_CODE);
      if Code < 0 then
        begin
          Text := Format('Cannot attach FCE (error %d)',[Code]);
          DisplayLine(WinFTP.eMessage, Text);
          exit
        end;
      {set log file}
      Code := fceSetString(0, FCE_SET_LOG_FILE, 'Ftp2_pgm.log');
      if Code < 0 then DisplayError(eMessage, Code);
      {show FCE version}
      Code := fceGetInteger(0,FCE_GET_VERSION);
      D3 := $0F AND Code;  Code := Code SHR 4;
      D2 := $0F AND Code;  Code := Code SHR 4;
      D1 := $0F AND Code;
      Build := fceGetInteger(0,FCE_GET_BUILD);
      Text := Format('FTP Client Engine Version %1d.%1d.%1d Build %d',
          [D1,D2,D3,Build]);
      DisplayLine(WinFTP.eMessage, Text);
      GetMem(ServerPtr, 64); StrPCopy(ServerPtr, eServer.Text);
      GetMem(UserPtr, 64); StrPCopy(UserPtr, eUser.Text);
      GetMem(PassPtr, 64); StrPCopy(PassPtr, ePass.Text);
      (* connect to FTP server *)
      DisplayLine(WinFTP.eMessage,Format('Connecting to %s ...',[eServer.Text]));
      SCREEN.Cursor := crHourGlass;
      Code := fceConnect(0, ServerPtr, UserPtr, PassPtr);
      FreeMem(ServerPtr,64); FreeMem(UserPtr,64); FreeMem(PassPtr,64);
      if Code >= 0 then
        begin
          {set PASSIVE mode}
          fceSetInteger(0, FCE_SET_PASSIVE, 1);
          GetMem(TempPtr,128);
          {get local directory}
          Code := fceGetLocalDir(0,TempPtr,128);
          if Code < 0 then DisplayError(eMessage, Code)
          else
            begin
              eLocalDir.Text := StrPas(TempPtr);
              DisplayLine(eMessage, Format('Local directory is %s', [eLocalDir.Text]));
            end;
          {get server directory}
          Code := fceGetServerDir(0,TempPtr,128);
          if Code < 0 then DisplayError(eMessage, Code)
          else
            begin
              eServerDir.Text := StrPas(TempPtr);
              DisplayLine(eMessage, Format('Server directory is %s', [eServerDir.Text]));
            end;
          FreeMem(TempPtr, 128);
          {connected}
          bFull.Enabled := True;
          bName.Enabled := True;
          bExit.Enabled := True;
          bLocalDir.Enabled := True;
          bServerDir.Enabled := True;
          bGet.Enabled := True;
          bPut.Enabled := True;
          bDel.Enabled := True;
          bChk.Enabled := True;
          bConnect.Caption := 'Quit';
          DisplayLine(eMessage, 'Connected.')
        end
      else
        begin
          DisplayError(eMessage, Code);
          fceRelease
        end;
      SCREEN.Cursor := crDefault
    end
end;

procedure TWinFTP.bExitClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TWinFTP.bClearClick(Sender: TObject);
begin
   eMessage.Lines.Clear
end;

procedure TWinFTP.bFullClick(Sender: TObject);
var
  Code : Integer;
  BufferPtr : PChar;
  BufferString : String;
begin
  GetMem(BufferPtr, BUFFER_SIZE);
  SCREEN.Cursor := crHourGlass;
  Code := fceGetList(0, FCE_FULL_LIST,BufferPtr, BUFFER_SIZE);
  SCREEN.Cursor := crDefault;
  if Code < 0 then
    begin
      DisplayError(eMessage, Code);
      FreeMem(BufferPtr, BUFFER_SIZE);
      exit
     end;
  if Code > 0 then
     begin
       BufferPtr[Code] := chr(0);
       {display contents of BufferPtr}
       DisplayPChar(eMessage, BufferPtr);
       {count lines on BufferPtr}
       Code := fceGetString(0, FCE_GET_LINE_COUNT, BufferPtr, BUFFER_SIZE);
       BufferString := Format('%d lines.',[Code]);
       DisplayLine(eMessage, BufferString);
     end;
   DisplayLine(eMessage,'[END]');
   FreeMem(BufferPtr, BUFFER_SIZE)
end;

procedure TWinFTP.bNameClick(Sender: TObject);
var
  Code : Integer;
  BufferPtr : PChar;
  BufferString : String;
begin
   GetMem(BufferPtr, BUFFER_SIZE);
   SCREEN.Cursor := crHourGlass;
   Code := fceGetList(0,FCE_NAME_LIST,BufferPtr,BUFFER_SIZE);
   SCREEN.Cursor := crDefault;
   if Code < 0 then
     begin
       DisplayError(eMessage, Code);
       FreeMem(BufferPtr, BUFFER_SIZE);
       exit
     end;
   if Code > 0 then
     begin
       BufferPtr[Code] := chr(0);
       {convert to string and display it}
       BufferString := StrPas(BufferPtr);
       DisplayLine(eMessage, BufferString);
     end;
   DisplayLine(eMessage,'[END]');
   FreeMem(BufferPtr, BUFFER_SIZE)
end;

procedure TWinFTP.bLocalDirClick(Sender: TObject);
var
  Code : Integer;
  TempPtr : PChar;
begin
  GetMem(TempPtr, 128);
  {set local directory}
  StrPCopy(TempPtr, eLocalDir.Text);
  fceSetLocalDir(0, TempPtr);
  {get local directory}
  Code := fceGetLocalDir(0,TempPtr,128);
  if Code < 0 then DisplayError(eMessage, Code)
  else
    begin
      eLocalDir.Text := StrPas(TempPtr);
      DisplayLine(eMessage, Format('Local directory is %s', [eLocalDir.Text]));
    end;
  FreeMem(TempPtr, 128)
end;

procedure TWinFTP.bServerDirClick(Sender: TObject);
var
  Code : Integer;
  TempPtr : PChar;
begin
  GetMem(TempPtr, 128);
  {set server directory}
  StrPCopy(TempPtr, eServerDir.Text);
  fceSetServerDir(0, TempPtr);
  {get server directory}
  Code := fceGetServerDir(0,TempPtr,128);
  if Code < 0 then DisplayError(eMessage, Code)
  else
    begin
       eServerDir.Text := StrPas(TempPtr);
       DisplayLine(eMessage, Format('Server directory is %s', [eServerDir.Text]));
    end;
  FreeMem(TempPtr, 128)
end;

procedure TWinFTP.oAsciiClick(Sender: TObject);
var
  Code : Integer;
begin
  Code := fceSetMode(0, 'A');
  if Code < 0 then DisplayError(eMessage, Code)
  else DisplayLine(eMessage,'XFER mode set to ASCII')
end;

procedure TWinFTP.oBinaryClick(Sender: TObject);
var
  Code : Integer;
begin
  Code := fceSetMode(0, 'B');
  if Code < 0 then DisplayError(eMessage, Code)
  else DisplayLine(eMessage,'XFER mode set to BINARY')
end;

procedure TWinFTP.bGetClick(Sender: TObject);
var
  Code : Integer;
  TempPtr : PChar;
begin
  {set filename}
  if Length(eFilename.Text) = 0 then
    begin
      DisplayLine(eMessage,'Missing Filename');
      exit
    end;
  GetMem(TempPtr, 128);
  StrPCopy(TempPtr, eFilename.Text);
  {get file}
  DisplayString(eMessage, 'Downloading ' + eFileName.Text);
  if oAscii.Checked then Displayline(eMessage, ' as ASCII')
  else DisplayLine(eMessage, ' as Binary');
  SCREEN.Cursor := crHourGlass;
  {turn AUTO_CALL off}
  Code := fceSetInteger(0, FCE_SET_AUTO_CALL_DRIVER, 0);
  {prepare to download file}
  Code := fceGetFile(0,TempPtr);
  if Code < 0 then DisplayError(eMessage, Code);
  FreeMem(TempPtr, 128);
  {run the driver to download the file}
  WinFtp.eStatus.Text := '0 bytes        ';
  Timer1.Enabled := True
end;

procedure TWinFTP.bPutClick(Sender: TObject);
var
  Code : Integer;
  TempPtr : PChar;
begin
  {set filename}
  if Length(eFilename.Text) = 0 then
    begin
      DisplayLine(eMessage,'Missing Filename');
      exit
    end;
  GetMem(TempPtr, 128);
  StrPCopy(TempPtr, eFilename.Text);
  {put file}
  DisplayString(eMessage, 'Uploading ' + eFileName.Text);
  if oAscii.Checked then Displayline(eMessage, ' as ASCII')
  else DisplayLine(eMessage, ' as Binary');
  SCREEN.Cursor := crHourGlass;
  {turn AUTO_CALL off}
  Code := fceSetInteger(0, FCE_SET_AUTO_CALL_DRIVER, 0);
  {prepare to upload file}
  Code := fcePutFile(0,TempPtr);
  if Code < 0 then DisplayError(eMessage, Code);
  FreeMem(TempPtr, 128);
  {run the driver to upload the file}
  WinFtp.eStatus.Text := '0 bytes        ';
  Timer1.Enabled := True
end;

procedure TWinFTP.bDelClick(Sender: TObject);
var
  Code : Integer;
  TempPtr : PChar;
begin
  {set filename}
  if Length(eFilename.Text) = 0 then
    begin
      DisplayLine(eMessage,'Missing Filename');
      exit
    end;
  GetMem(TempPtr, 128);
  StrPCopy(TempPtr, eFilename.Text);
  {delete file}
  DisplayLine(eMessage, 'Deleting ' + eFileName.Text);
  SCREEN.Cursor := crHourGlass;
  Code := fceDelFile(0,TempPtr);
  if Code < 0 then DisplayError(eMessage, Code)
  else DisplayLine(eMessage, 'Done');
  SCREEN.Cursor := crDefault;
  FreeMem(TempPtr, 128)
end;


procedure TWinFTP.bChkClick(Sender: TObject);
var
  Code : Integer;
  TempPtr : PChar;
begin
  if Length(eFilename.Text) = 0 then
    begin
      DisplayLine(eMessage,'Missing Filename');
      exit
    end;
  GetMem(TempPtr, 128);
  StrPCopy(TempPtr, eFilename.Text);
  SCREEN.Cursor := crHourGlass;
  Code := fceGetList(0,FCE_NAME_LIST_FILE,TempPtr,100);
  SCREEN.Cursor := crDefault;
  if Code <= 0 then DisplayLine(eMessage,'File does NOT exist.')
  else DisplayLine(eMessage,'File exists.');
  FreeMem(TempPtr, 128)
end;

procedure TWinFTP.Timer1Timer(Sender: TObject);
var
  I : Integer;
  Code : Integer;
  Bytes: LongInt;
  Text : String;
begin
  {call the driver}
  for I := 1 To 25 do
    begin
      Code := fceDriver(0);
      if Code <=0 then break
    end;
  {get the byte count}
  Bytes := fceGetInteger(0, FCE_GET_FILE_BYTES_RCVD) +
           fceGetInteger(0, FCE_GET_FILE_BYTES_SENT);
  Text := Format('%d bytes',[Bytes]);
  WinFtp.eStatus.Text := Text;
  {check return status from fceDriver}
  if Code <=0 then
    begin
      Timer1.Enabled := false;
      SCREEN.Cursor := crDefault;
      {turn AUTO_CALL back on}
      Code := fceSetInteger(0, FCE_SET_AUTO_CALL_DRIVER, 1);
      DisplayLine(eMessage, 'File xfer complete.')
     end
end;

end.
