unit Ftpw_pgm;

interface

uses
  fcew, fce, keycode, displayu,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls;

type
  TWinFTP = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    eServer: TEdit;
    eUser: TEdit;
    ePass: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    eLocalDir: TEdit;
    Label7: TLabel;
    Label8: TLabel;
    eServerDir: TEdit;
    Label9: TLabel;
    bConnect: TButton;
    bExit: TButton;
    bLocalDir: TButton;
    bServerDir: TButton;
    bName: TButton;
    bFull: TButton;
    Label10: TLabel;
    oAscii: TRadioButton;
    oBinary: TRadioButton;
    Label12: TLabel;
    eFilename: TEdit;
    Label13: TLabel;
    bGet: TButton;
    bPut: TButton;
    bDel: TButton;
    bChk: TButton;
    eMessage: TMemo;
    bClear: TButton;
    procedure bConnectClick(Sender: TObject);
    procedure bExitClick(Sender: TObject);
    procedure bClearClick(Sender: TObject);
    procedure bFullClick(Sender: TObject);
    procedure bNameClick(Sender: TObject);
    procedure bLocalDirClick(Sender: TObject);
    procedure bServerDirClick(Sender: TObject);
    procedure oAsciiClick(Sender: TObject);
    procedure oBinaryClick(Sender: TObject);
    procedure bGetClick(Sender: TObject);
    procedure bPutClick(Sender: TObject);
    procedure bDelClick(Sender: TObject);
    procedure bChkClick(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

const
   BUFFER_SIZE = 15000;

var
  WinFTP: TWinFTP;

implementation

{$R *.DFM}

procedure TWinFTP.bConnectClick(Sender: TObject);
var
  Code : Integer;
  Build : Integer;
  D1, D2, D3 : Integer;
  Text : String;
begin
  {use bExit as "connected" flag}
  if not bExit.Enabled then
    begin
      {close existing connection}
      Code := fClose(0);
      if Code < 0 then DisplayError(eMessage, Code);
      fRelease;
      {ready for new connection}
      bConnect.Caption := 'CONNECT';
      bExit.Enabled := True
    end
  else
    begin
      {do we have all necessary parameters ?}
      if Length(eServer.Text) = 0 then
      begin
        DisplayLine(eMessage,'FTP server name is missing');
        exit;
      end;
      if Length(eUser.Text) = 0 then
      begin
        DisplayLine(eMessage,'FTP user name is missing');
        exit;
      end;
      if Length(ePass.Text) = 0 then
      begin
        DisplayLine(eMessage,'FTP password is missing');
        exit;
      end;
      {attach FCE}
      Code := fAttach(1, FCE_KEY_CODE);
      if Code < 0 then
        begin
          Text := Format('Cannot attach FCE (error %d)',[Code]);
          DisplayLine(WinFTP.eMessage, Text);
          exit
        end;
      {set log file}
      Code := fSetString(0, FCE_SET_LOG_FILE, 'ftpw_pgm.log');
      if Code < 0 then DisplayError(eMessage, Code);
      {show FCE version}
      Code := fGetInteger(0,FCE_GET_VERSION);
      D3 := $0F AND Code;  Code := Code SHR 4;
      D2 := $0F AND Code;  Code := Code SHR 4;
      D1 := $0F AND Code;
      Build := fGetInteger(0,FCE_GET_BUILD);
      Text := Format('FTP Client Engine Version %1d.%1d.%1d Build %d',
           [D1,D2,D3,Build]);
      DisplayLine(WinFTP.eMessage, Text);
      (* connect to FTP server *)
      DisplayLine(WinFTP.eMessage,Format('Connecting to %s ...',[eServer.Text]));
      SCREEN.Cursor := crHourGlass;
      Code := fConnect(0, eServer.Text, eUser.Text, ePass.Text);
      if Code >= 0 then
        begin
          {set PASSIVE mode}
          fSetInteger(0, FCE_SET_PASSIVE, 1);
          {get local directory}
          Code := fGetLocalDir(0, Text);
          if Code < 0 then DisplayError(eMessage, Code)
          else
            begin
              eLocalDir.Text := Text;
              DisplayLine(eMessage,
                Format('Local directory is %s', [eLocalDir.Text]) )
            end;
          {get server directory}
          Code := fGetServerDir(0, Text);
          if Code < 0 then DisplayError(eMessage, Code)
          else
            begin
              eServerDir.Text := Text;
              DisplayLine(eMessage,
                Format('Server directory is %s', [eServerDir.Text]) )
            end;
          {all done}
          bExit.Enabled := False;
          bConnect.Caption := 'Quit';
          DisplayLine(eMessage, 'Connected.')
        end
      else
        begin
          DisplayError(eMessage, Code);
          fceRelease
        end;
      SCREEN.Cursor := crDefault
   end
end;

procedure TWinFTP.bExitClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TWinFTP.bClearClick(Sender: TObject);
begin
   eMessage.Lines.Clear
end;


{NOTE: Beginning with Delphi 2, strings can be a maximum of 2GB rather
       than the 256 bytes in Delphi 1. However, there is an error in the
       Delphi "StrPCopy" function in Delphi versions 2 and 3. The StrPCopy
       function will not copy more than 256 bytes.

       This means that fGetList cannot return more than 255 byte strings
       in Delphi versions 1, 2, and 3. See FTP_PGM.PAS in which does not
       use the FCEW.PAS unit.

 *** IMPORTANT ***

       If you are running Delphi 4 or above, uncomment the
       "$DEFINE DELPHI4_OR_BETTER" statement below by removing the blank
       (space) in front of the "$DEFINE".
}

{ $DEFINE DELPHI4_OR_BETTER}

procedure GetList(ListType : Integer);
{$IFDEF DELPHI4_OR_BETTER}
var
  Code : Integer;
  Text : String;
begin
   SCREEN.Cursor := crHourGlass;
   Code := fGetList(0, ListType, Text);
   SCREEN.Cursor := crDefault;
   if Code < 0 then
     begin
       DisplayError(Winftp.eMessage, Code);
       exit
     end;
   if Code > 0 then
     begin
       {display it}
       DisplayLine(WinftP.eMessage,Text);
     end;
   DisplayLine(Winftp.eMessage,'[END]');
{$ELSE}
const
   BUFFER_SIZE = 15000;
var
  Code : Integer;
  BufferPtr : PChar;
  Text : String;
begin
  GetMem(BufferPtr, BUFFER_SIZE);
  SCREEN.Cursor := crHourGlass;
  Code := fceGetList(0, FCE_FULL_LIST,BufferPtr, BUFFER_SIZE);
  SCREEN.Cursor := crDefault;
  if Code < 0 then
    begin
      DisplayError(Winftp.eMessage, Code);
      FreeMem(BufferPtr, BUFFER_SIZE);
      exit
     end;
  if Code > 0 then
     begin
       BufferPtr[Code] := chr(0);
       {display contents of BufferPtr}
       DisplayPChar(Winftp.eMessage, BufferPtr);
       {count lines on BufferPtr}
       Code := fceGetString(0, FCE_GET_LINE_COUNT, BufferPtr, BUFFER_SIZE);
       Text := Format('%d lines.',[Code]);
       DisplayLine(Winftp.eMessage, Text);
     end;
   DisplayLine(Winftp.eMessage,'[END]');
   FreeMem(BufferPtr, BUFFER_SIZE)
{$ENDIF}
end;

procedure TWinFTP.bFullClick(Sender: TObject);
begin
  GetList(FCE_FULL_LIST)
end;

procedure TWinFTP.bNameClick(Sender: TObject);
begin
  GetList(FCE_NAME_LIST)
end;

procedure TWinFTP.bLocalDirClick(Sender: TObject);
var
  Code : Integer;
  Text : String;
begin
  {set local directory}
  Code := fSetLocalDir(0, eLocalDir.Text);
  if Code < 0 then DisplayError(eMessage, Code)
  else
    begin
      {get local directory}
      Code := fGetLocalDir(0, Text);
      if Code < 0 then DisplayError(eMessage, Code)
      else
        begin
          eLocalDir.Text := Text;
          DisplayLine(eMessage, Format('Local directory is %s', [eLocalDir.Text]))
        end;
    end;
end;

procedure TWinFTP.bServerDirClick(Sender: TObject);
var
  Code : Integer;
  Text : String;
begin
  {set server directory}
  Code := fSetServerDir(0, eServerDir.Text);
  if Code < 0 then DisplayError(eMessage, Code)
  else
    begin
      {get server directory}
      Code := fGetServerDir(0,Text);
      if Code < 0 then DisplayError(eMessage, Code)
      else
        eServerDir.Text := Text;
        DisplayLine(eMessage, Format('Server directory is %s', [eServerDir.Text]))
      end;
end;

procedure TWinFTP.oAsciiClick(Sender: TObject);
var
  Code : Integer;
begin
  Code := fSetMode(0, 'A');
  if Code < 0 then DisplayError(eMessage, Code)
  else DisplayLine(eMessage,'XFER mode set to ASCII')
end;

procedure TWinFTP.oBinaryClick(Sender: TObject);
var
  Code : Integer;
begin
  Code := fSetMode(0, 'B');
  if Code < 0 then DisplayError(eMessage, Code)
  else DisplayLine(eMessage,'XFER mode set to BINARY')
end;

procedure TWinFTP.bGetClick(Sender: TObject);
var
  Code : Integer;
begin
  {set filename}
  if Length(eFilename.Text) = 0 then
    begin
      DisplayLine(eMessage,'Missing Filename');
      exit
    end;
  {get file}
  DisplayString(eMessage, 'Downloading ' + eFileName.Text);
  if oAscii.Checked then Displayline(eMessage, ' as ASCII')
  else DisplayLine(eMessage, ' as Binary');
  SCREEN.Cursor := crHourGlass;
  Code := fGetFile(0, eFilename.Text);
  if Code < 0 then DisplayError(eMessage, Code)
  else DisplayLine(eMessage, 'Done');
  SCREEN.Cursor := crDefault;
end;

procedure TWinFTP.bPutClick(Sender: TObject);
var
  Code : Integer;
begin
  {set filename}
  if Length(eFilename.Text) = 0 then
    begin
      DisplayLine(eMessage,'Missing Filename');
      exit
    end;
  {put file}
  DisplayString(eMessage, 'Uploading ' + eFileName.Text);
  if oAscii.Checked then Displayline(eMessage, ' as ASCII')
  else DisplayLine(eMessage, ' as Binary');
  SCREEN.Cursor := crHourGlass;
  Code := fPutFile(0, eFilename.Text);
  if Code < 0 then DisplayError(eMessage, Code)
  else DisplayLine(eMessage, 'Done');
  SCREEN.Cursor := crDefault
end;

procedure TWinFTP.bDelClick(Sender: TObject);
var
  Code : Integer;
begin
  {set filename}
  if Length(eFilename.Text) = 0 then
    begin
      DisplayLine(eMessage,'Missing Filename');
      exit
    end;
  {delete file}
  DisplayLine(eMessage, 'Deleting ' + eFileName.Text);
  SCREEN.Cursor := crHourGlass;
  Code := fDelFile(0, eFilename.Text);
  if Code < 0 then DisplayError(eMessage, Code)
  else DisplayLine(eMessage, 'Done');
  SCREEN.Cursor := crDefault
end;


procedure TWinFTP.bChkClick(Sender: TObject);
var
  Code : Integer;
  Text : String;
begin
  if Length(eFilename.Text) = 0 then
    begin
      DisplayLine(eMessage,'Missing Filename');
      exit
    end;
  SCREEN.Cursor := crHourGlass;
  Text := eFilename.Text;
  Code := fGetList(0,FCE_NAME_LIST_FILE, Text);
  SCREEN.Cursor := crDefault;
  if Code <= 0 then DisplayLine(eMessage,'File does NOT exist.')
  else DisplayLine(eMessage,'File exists.')
end;

end.
